#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "math/Complex.h"

#define SCREEN_WIDTH    1280
#define SCREEN_HEIGHT   720
#define MAX_ITERATIONS  40
#define TOLERANCE       1e-6

// the function
Complex f(Complex z)
{
    // z^3 - 1
    return (z * z * z) - Complex(1, 0);
}

// derivative of the function
Complex df(Complex z)
{
    // 3z^2
    return (z * z) * 3;
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Newton relaxed 5", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    static double zoom = 0.1 / (double)SCREEN_HEIGHT;
    static double centerX = (double)SCREEN_WIDTH  / 2.0;
    static double centerY = (double)SCREEN_HEIGHT / 2.0;

    Complex a(2, 0);

    // draw the fractal
    for (int y = 0; y < SCREEN_HEIGHT; y++)
    {
        for (int x = 0; x < SCREEN_WIDTH; x++)
        {
            // convert the pixel pos to a complex number
            Complex z;
            z.re = (x - centerX) * zoom;
            z.im = (y - centerY) * zoom;

            // iterate
            int iter;

            for (iter = 0; iter < MAX_ITERATIONS; iter++)
                z = z - a * (f(z) / df(z));

            // draw the pixel
            Color color;
            color = Color(0, 0, 0);

            float angle = z.arg();
            color.r = 127 * (1 + sin(angle + 7 * M_PI / 6));
            color.g = 127 * (1 + sin(angle + 3 * M_PI / 6));
            color.b = 127 * (1 + sin(angle - 1 * M_PI / 6));

            gfx.setPixel(x, y, color);
        }

        gfx.render();
        sys.processEvents();

        if (sys.isQuitRequested() == true)
            break;
    }

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        gfx.render();
        sys.processEvents();
        sys.wait(16);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
